@@ -77,7 +77,8 @@ gdk_x11_display_manager_set_default_display (GdkDisplayManager *manager,
 {
   GDK_X11_DISPLAY_MANAGER (manager)->default_display = display;
 
-  _gdk_x11_display_make_default (display);
+  if (display)
+    _gdk_x11_display_make_default (display);
 }
 
 static void
@@ -109,7 +110,9 @@ gdk_x11_display_manager_class_init (GdkX11DisplayManagerClass *class)
   manager_class->get_atom_name = _gdk_x11_display_manager_get_atom_name;
   manager_class->lookup_keyval = _gdk_x11_display_manager_lookup_keyval;
   manager_class->get_keyval_name = _gdk_x11_display_manager_get_keyval_name;
+#ifdef HAVE_XCONVERTCASE
   manager_class->keyval_convert_case = _gdk_x11_display_manager_keyval_convert_case;
+#endif
 }
 
 void
