@@ -61,7 +61,7 @@ _gtk_css_parser_new (const char            *data,
   parser->user_data = user_data;
 
   parser->line_start = data;
-  parser->line = 1;
+  parser->line = 0;
 
   return parser;
 }
@@ -386,7 +386,10 @@ _gtk_css_parser_read_string (GtkCssParser *parser)
   quote = *parser->data;
   
   if (quote != '"' && quote != '\'')
-    return NULL;
+    {
+      _gtk_css_parser_error (parser, "Expected a string.");
+      return NULL;
+    }
   
   parser->data++;
   str = g_string_new (NULL);
@@ -659,7 +662,7 @@ gtk_css_parser_read_symbolic_color_function (GtkCssParser *parser,
           child2 = _gtk_css_parser_read_symbolic_color (parser);
           if (child2 == NULL)
             {
-              g_object_unref (child1);
+              gtk_symbolic_color_unref (child1);
               return NULL;
             }
         }
@@ -716,6 +719,7 @@ gtk_css_parser_read_symbolic_color_function (GtkCssParser *parser,
 
   if (!_gtk_css_parser_try (parser, ")", TRUE))
     {
+      _gtk_css_parser_error (parser, "Expected ')' in color definition");
       gtk_symbolic_color_unref (symbolic);
       return NULL;
     }
@@ -871,17 +875,20 @@ _gtk_css_parser_resync_internal (GtkCssParser *parser,
       case '(':
         parser->data++;
         _gtk_css_parser_resync (parser, FALSE, ')');
-        parser->data++;
+        if (*parser->data)
+          parser->data++;
         break;
       case '[':
         parser->data++;
         _gtk_css_parser_resync (parser, FALSE, ']');
-        parser->data++;
+        if (*parser->data)
+          parser->data++;
         break;
       case '{':
         parser->data++;
         _gtk_css_parser_resync (parser, FALSE, '}');
-        parser->data++;
+        if (*parser->data)
+          parser->data++;
         if (sync_at_semicolon || !terminator)
           {
             _gtk_css_parser_skip_whitespace (parser);
@@ -898,6 +905,8 @@ _gtk_css_parser_resync_internal (GtkCssParser *parser,
           }
         parser->data++;
         continue;
+      case '\0':
+        break;
       case '/':
       default:
         parser->data++;
