@@ -36,7 +36,6 @@
 
 G_BEGIN_DECLS
 
-typedef struct _GtkIconFactoryClass GtkIconFactoryClass;
 
 #define GTK_TYPE_ICON_FACTORY              (gtk_icon_factory_get_type ())
 #define GTK_ICON_FACTORY(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), GTK_TYPE_ICON_FACTORY, GtkIconFactory))
@@ -47,11 +46,15 @@ typedef struct _GtkIconFactoryClass GtkIconFactoryClass;
 #define GTK_TYPE_ICON_SET                  (gtk_icon_set_get_type ())
 #define GTK_TYPE_ICON_SOURCE               (gtk_icon_source_get_type ())
 
+typedef struct _GtkIconFactoryPrivate       GtkIconFactoryPrivate;
+typedef struct _GtkIconFactoryClass         GtkIconFactoryClass;
+
 struct _GtkIconFactory
 {
   GObject parent_instance;
 
-  GHashTable *GSEAL (icons);
+  /*< private >*/
+  GtkIconFactoryPrivate *priv;
 };
 
 struct _GtkIconFactoryClass
@@ -65,12 +68,6 @@ struct _GtkIconFactoryClass
   void (*_gtk_reserved4) (void);
 };
 
-#ifdef G_OS_WIN32
-/* Reserve old names for DLL ABI backward compatibility */
-#define gtk_icon_source_set_filename gtk_icon_source_set_filename_utf8
-#define gtk_icon_source_get_filename gtk_icon_source_get_filename_utf8
-#endif
-
 GType           gtk_icon_factory_get_type (void) G_GNUC_CONST;
 GtkIconFactory* gtk_icon_factory_new      (void);
 void            gtk_icon_factory_add      (GtkIconFactory *factory,
@@ -112,7 +109,7 @@ GtkIconSize           gtk_icon_size_register       (const gchar *name,
 void                  gtk_icon_size_register_alias (const gchar *alias,
                                                     GtkIconSize  target);
 GtkIconSize           gtk_icon_size_from_name      (const gchar *name);
-G_CONST_RETURN gchar* gtk_icon_size_get_name       (GtkIconSize  size);
+const gchar*          gtk_icon_size_get_name       (GtkIconSize  size);
 
 /* Icon sets */
 
@@ -124,17 +121,19 @@ GtkIconSet* gtk_icon_set_ref             (GtkIconSet      *icon_set);
 void        gtk_icon_set_unref           (GtkIconSet      *icon_set);
 GtkIconSet* gtk_icon_set_copy            (GtkIconSet      *icon_set);
 
+#if !defined (GTK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION)
 /* Get one of the icon variants in the set, creating the variant if
  * necessary.
  */
+GDK_DEPRECATED_FOR(gtk_icon_set_render_icon_pixbuf)
 GdkPixbuf*  gtk_icon_set_render_icon     (GtkIconSet      *icon_set,
                                           GtkStyle        *style,
                                           GtkTextDirection direction,
                                           GtkStateType     state,
                                           GtkIconSize      size,
                                           GtkWidget       *widget,
-                                          const char      *detail);
-
+                                          const gchar     *detail);
+#endif
 
 void           gtk_icon_set_add_source   (GtkIconSet          *icon_set,
                                           const GtkIconSource *source);
@@ -155,9 +154,9 @@ void           gtk_icon_source_set_icon_name            (GtkIconSource       *so
 void           gtk_icon_source_set_pixbuf               (GtkIconSource       *source,
                                                          GdkPixbuf           *pixbuf);
 
-G_CONST_RETURN gchar* gtk_icon_source_get_filename  (const GtkIconSource *source);
-G_CONST_RETURN gchar* gtk_icon_source_get_icon_name (const GtkIconSource *source);
-GdkPixbuf*            gtk_icon_source_get_pixbuf    (const GtkIconSource *source);
+const gchar *    gtk_icon_source_get_filename             (const GtkIconSource *source);
+const gchar *    gtk_icon_source_get_icon_name            (const GtkIconSource *source);
+GdkPixbuf*       gtk_icon_source_get_pixbuf               (const GtkIconSource *source);
 
 void             gtk_icon_source_set_direction_wildcarded (GtkIconSource       *source,
                                                            gboolean             setting);
